@@ -12,7 +12,7 @@ summary_fun <- function(x){
     print(param_summary[1:2,],row.names = TRUE,digits=3,right=FALSE)
     cat("\nHyperparameters:\n")
     print(param_summary[3:nrow(param_summary),],row.names = TRUE,digits=3,right=FALSE)
-    cat("\nDIC:",x$DIC)
+    cat("\nWAIC:",x$WAIC)
 }
 
 #' Custom bdrc theme
@@ -24,18 +24,18 @@ summary_fun <- function(x){
 #' @keywords internal
 theme_bdrc <- function(...,scaling=1){
     title_size <- scaling*12
-    text_size <- scaling*10
-    plot_title_size=scaling*15
+    text_size <- scaling*12
+    plot_title_size=scaling*12
     theme_classic() %+replace%
         theme( #text = element_text(family="Times", face="plain"),
                strip.background = element_blank(),
                strip.text.x = element_text(size = title_size),
-               axis.title.x = element_text(size=title_size),
-               axis.title.y = element_text(size=title_size,angle=90),
-               axis.text.x = element_text(size=text_size),
-               axis.text.y = element_text(size=text_size),
-               legend.text = element_text(size=text_size),
-               legend.title = element_text(size=text_size),
+               axis.title.x = element_text(size = title_size),
+               axis.title.y = element_text(size = title_size,angle=90),
+               axis.text.x = element_text(size = text_size),
+               axis.text.y = element_text(size = text_size),
+               legend.text = element_text(size = text_size),
+               legend.title = element_text(size = text_size),
                plot.title = element_text(size=plot_title_size),
                panel.border = element_rect(colour="black",fill=NA),
                ...)
@@ -77,13 +77,14 @@ histogram_breaks <-function(x){
 #' @param param a character vector with the parameters to plot. Defaults to NULL and is only used if type is "trace" or "histogram". Allowed values are the parameters given in the model summary of x as well as "hyperparameters" or "latent_parameters" for specific groups of parameters.
 #' @param transformed a logical value indicating whether the quantity should be plotted on a transformed scale used during the Bayesian inference. Defaults to FALSE.
 #' @param title a character denoting the title of the plot. Defaults to NULL, i.e. no title.
+#' @param xlim numeric vector of length 2, denoting the limits on the x axis of the plot. Applicable for types "rating_curve","rating_curve_mean","f","beta","sigma_eps","residuals".
+#' @param ylim numeric vector of length 2, denoting the limits on the y axis of the plot. Applicable for types "rating_curve","rating_curve_mean","f","beta","sigma_eps","residuals".
 #' @return returns an object of class ggplot2.
-#' @importFrom ggplot2 ggplot aes geom_point geom_path geom_histogram geom_abline geom_hline geom_smooth facet_wrap scale_color_manual scale_x_continuous scale_y_continuous expansion label_parsed ggtitle xlab ylab
+#' @importFrom ggplot2 ggplot aes geom_point geom_path geom_histogram geom_abline geom_hline geom_smooth facet_wrap scale_color_manual scale_x_continuous scale_y_continuous expansion label_parsed ggtitle xlab ylab geom_blank margin element_text theme
 #' @importFrom rlang .data
 #' @importFrom stats median
 #' @keywords internal
-plot_fun <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
-    args <- list(...)
+plot_fun <- function(x,type='rating_curve',param=NULL,transformed=FALSE,title=NULL,xlim=NULL,ylim=NULL,...){
     color_palette <- c("green","red","slateblue1","hotpink","#56B4E9","#E69F00","#000000","#999999","#CC79A7","#D55E00","#0072B2","#009E73")
     legal_types <- c('rating_curve','rating_curve_mean','f','beta','sigma_eps','residuals','trace','histogram','r_hat','autocorrelation')
     if(!(type %in% legal_types)){
@@ -106,21 +107,23 @@ plot_fun <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
             plot_dat$name_expr <- factor(plot_dat$name,levels=param_levels,labels=sapply(param_levels,get_param_expression))
             plot_dat$chain <- factor(as.character(plot_dat$chain),levels=1:max(plot_dat$chain))
             p <- ggplot(plot_dat,aes(x=.data$iter,y=.data$value,col=.data$chain)) +
-                geom_path(alpha=0.7,size=0.3) +
+                geom_path(alpha=0.7,linewidth=0.3) +
                 facet_wrap(~name_expr,scales='free',labeller = label_parsed) +
                 scale_color_manual(values=c("#BC3C29FF","#0072B5FF","#E18727FF","#20854EFF"),
                                    name='Chain number') +
                 xlab('Iteration') +
                 ylab('') +
+                ggtitle(if(!is.null(title)) title else "Traceplots") +
                 theme_bdrc()
         }else{
             param_expr <- get_param_expression(params)
             plot_dat$chain_name <- paste0('Chain nr ',plot_dat$chain)
             p <- ggplot(plot_dat,aes(x=.data$iter,y=.data$value)) +
-                geom_path(col="#0072B5FF",size=0.3) +
+                geom_path(col="#0072B5FF",linewidth=0.3) +
                 facet_wrap(~chain_name,scales='free') +
                 xlab('Iteration') +
                 ylab(parse(text=param_expr)) +
+                ggtitle(if(!is.null(title)) title else "Traceplots") +
                 theme_bdrc()
         }
     }else if(type=='histogram'){
@@ -134,12 +137,16 @@ plot_fun <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
         plot_dat$chain <- factor(as.character(plot_dat$chain),levels=1:max(plot_dat$chain))
         p <- ggplot(plot_dat,aes(x=.data$value)) +
             geom_histogram(bins=50,fill="#0072B5FF") +
-            facet_wrap(~name_expr,scales='free',labeller=label_parsed) +
+            facet_wrap(~name_expr,scales='free',labeller=label_parsed,strip.position = "bottom") +
             scale_x_continuous(breaks=histogram_breaks) +
             scale_y_continuous(expand=c(0,0,0.05,0)) +
             xlab('') +
             ylab('') +
-            theme_bdrc()
+            ggtitle(if(!is.null(title)) title else if( length(param)==1 ) "Histogram of posterior draws" else "Histograms of posterior draws" ) +
+            theme_bdrc() +
+            theme(plot.title = element_text( vjust = 2 ),
+                  strip.placement = "outside",
+                  plot.margin =  ggplot2::margin(t = 10, r = 10, b = -10, l = 0, unit = "pt"))
     }else if(type=='rating_curve' | type=='rating_curve_mean'){
         if(transformed){
             x_lab <- "paste('','',log,,,,'(','',italic(paste('h-',hat(paste('c')))),')','','')"
@@ -158,11 +165,13 @@ plot_fun <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
                 geom_path(aes(x=.data$`log(h-c_hat)`,y=.data$log_lower),linetype='dashed',alpha=0.95) +
                 geom_path(aes(x=.data$`log(h-c_hat)`,y=.data$log_upper),linetype='dashed',alpha=0.95) +
                 geom_point(data=plot_dat[!is.na(plot_dat$log_Q),],aes(x=.data$`log(h-c_hat)`,y=.data$log_Q), size=.9, shape=21, fill="gray60", color="black",alpha=0.95) +
-                scale_x_continuous(limits=if(!is.null(args$xlim)) args$xlim else c(NA,NA),expand=c(0.01,0)) +
-                scale_y_continuous(limits=if(!is.null(args$ylim)) args$ylim else c(NA,NA),expand=c(0.01,0)) +
+                scale_x_continuous(limits=if(!is.null(xlim)) xlim else c(NA,NA),expand=c(0.01,0)) +
+                scale_y_continuous(limits=if(!is.null(ylim)) ylim else c(NA,NA),expand=c(0.01,0)) +
                 xlab(parse(text=x_lab)) +
                 ylab(parse(text=y_lab)) +
-                theme_bdrc()
+                ggtitle(if(!is.null(title)) title else "Log-transformed rating curve") +
+                theme_bdrc() +
+                theme(plot.title = element_text( vjust = 2 ))
         }else{
             x_lab <- "paste('','',italic(paste('Q')),paste('['),italic(paste('m',phantom() ^ {paste('3')},'/s')),paste(']'),'')"
             y_lab <- "paste('','',italic(paste('h')),paste('['),italic(paste('m')),paste(']'),'')"
@@ -171,12 +180,14 @@ plot_fun <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
                 geom_path(aes(x=.data$lower,y=.data$h),linetype='dashed',alpha=0.95) +
                 geom_path(aes(x=.data$upper,y=.data$h),linetype='dashed',alpha=0.95) +
                 geom_point(data=x$data,aes(.data[[all.vars(x$formula)[1]]],.data[[all.vars(x$formula)[2]]]), size=.9, shape=21, fill="gray60", color="black",alpha=0.95) +
-                scale_x_continuous(limits=if(!is.null(args$xlim)) args$xlim else c(NA,NA),expand=expansion(mult=0.01)) +
-                #scale_x_continuous(limits=if(!is.null(args$xlim)) args$xlim else c(0,max(x$rating_curve$upper,x$data$Q)),expand=c(0.01,0)) +
-                scale_y_continuous(limits=if(!is.null(args$ylim)) args$ylim else c(NA,NA),expand=c(0.01,0)) +
+                scale_x_continuous(limits=if(!is.null(xlim)) xlim else c(NA,NA),expand=expansion(mult=0.01)) +
+                #scale_x_continuous(limits=if(!is.null(xlim)) xlim else c(0,max(x$rating_curve$upper,x$data$Q)),expand=c(0.01,0)) +
+                scale_y_continuous(limits=if(!is.null(ylim)) ylim else c(NA,NA),expand=c(0.01,0)) +
                 xlab(parse(text=x_lab)) +
                 ylab(parse(text=y_lab)) +
-                theme_bdrc()
+                ggtitle(if(!is.null(title)) title else "Rating curve") +
+                theme_bdrc() +
+                theme(plot.title = element_text( vjust = 2 ))
         }
     }else if(type=='sigma_eps'){
         x_lab <- "paste('','',italic(paste('h')),paste('['),italic(paste('m')),paste(']'),'')"
@@ -197,9 +208,11 @@ plot_fun <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
             geom_path(aes(x=.data$h,y=.data$upper),linetype='dashed') +
             xlab(parse(text=x_lab)) +
             ylab(parse(text=y_lab)) +
-            scale_x_continuous(limits=if(!is.null(args$xlim)) args$xlim else c(NA,NA),expand=c(0,0)) +
-            scale_y_continuous(limits=if(!is.null(args$ylim)) args$ylim else c(0,max(plot_dat$upper)*1.1),expand=c(0,0)) +
-            theme_bdrc()
+            scale_x_continuous(limits=if(!is.null(xlim)) xlim else c(NA,NA),expand=c(0,0)) +
+            scale_y_continuous(limits=if(!is.null(ylim)) ylim else c(0,max(plot_dat$upper)*1.1),expand=c(0,0)) +
+            ggtitle(if(!is.null(title)) title else "Std. dev. of the error terms") +
+            theme_bdrc() +
+            theme(plot.title = element_text( vjust = 2 ))
     }else if(type=='beta'){
         if(!('beta_summary' %in% names(x))){
             stop('Plots of type "beta" are only for models with stage dependent power law exponent, s.a. "gplm0" and "gplm"')
@@ -213,9 +226,11 @@ plot_fun <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
             geom_path(aes(.data$h,.data$upper),linetype='dashed') +
             xlab(parse(text=x_lab)) +
             ylab(parse(text=y_lab)) +
-            scale_x_continuous(if(!is.null(args$xlim)) args$xlim else c(NA,NA),expand=c(0,0)) +
-            scale_y_continuous(limits=if(!is.null(args$ylim)) args$ylim else c(NA,NA),expand=c(0,0)) +
-            theme_bdrc()
+            scale_x_continuous(if(!is.null(xlim)) xlim else c(NA,NA),expand=c(0,0)) +
+            scale_y_continuous(limits=if(!is.null(ylim)) ylim else c(NA,NA),expand=expansion(mult=rep(.05,2))) +
+            ggtitle(if(!is.null(title)) title else "Power-law exponent deviations") +
+            theme_bdrc() +
+            theme(plot.title = element_text( vjust = 2 ))
     }else if(type=='f'){
         x_lab <- "paste('','',italic(paste('h')),paste('['),italic(paste('m')),paste(']'),'')"
         h_in_data <- x$data[,all.vars(x$formula)[2],drop=TRUE]
@@ -235,9 +250,11 @@ plot_fun <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
             geom_path(aes(.data$h,.data$upper),linetype='dashed') +
             xlab(parse(text=x_lab)) +
             ylab(parse(text=y_lab)) +
-            scale_x_continuous(limits=if(!is.null(args$xlim)) args$xlim else c(NA,NA),expand=c(0,0)) +
-            scale_y_continuous(limits=if(!is.null(args$ylim)) args$ylim else c(min(1,0.9*min(plot_dat$lower)),max(3.5,1.1*max(plot_dat$upper))),expand=c(0,0)) +
-            theme_bdrc()
+            scale_x_continuous(limits=if(!is.null(xlim)) xlim else c(NA,NA),expand=c(0,0)) +
+            scale_y_continuous(limits=if(!is.null(ylim)) ylim else c(min(1,0.9*min(plot_dat$lower)),max(3.5,1.1*max(plot_dat$upper))),expand=c(0,0)) +
+            ggtitle(if(!is.null(title)) title else "Power-law exponent") +
+            theme_bdrc() +
+            theme(plot.title = element_text( vjust = 2 ))
     }else if(type=='residuals'){
         resid_dat <- get_residuals_dat(x)
         y_lab <- "paste('','log','(','',italic(paste('Q')),')','','-log','(','',italic(paste('',hat(paste('Q')))),')','','')"
@@ -245,17 +262,24 @@ plot_fun <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
         method <- 'loess'
         span <- 0.3
         p <- ggplot(data=resid_dat) +
-            geom_hline(yintercept=0,size=0.8,alpha=.95) +
+            geom_hline(yintercept=0,linewidth=0.8,alpha=.95) +
             geom_point(data=resid_dat[!is.na(resid_dat$Q),],aes(.data$`log(h-c_hat)`,.data$r_median), size=.9, shape=21, fill="gray60", color="black",alpha=0.95) +
-            geom_smooth(aes(x=.data$`log(h-c_hat)`,y=.data$r_upper),span=span,se=FALSE,stat = "smooth",color='black',linetype='dashed',size=0.5,alpha=0.95,method=method,formula='y~x') +
-            geom_smooth(aes(x=.data$`log(h-c_hat)`,y=.data$r_lower),span=span,se=FALSE,stat = "smooth",color='black',linetype='dashed',size=0.5,alpha=0.95,method=method,formula='y~x') +
-            geom_smooth(aes(x=.data$`log(h-c_hat)`,y=.data$m_upper),span=span,se=FALSE,stat = "smooth",color='black',linetype='solid',size=0.3,alpha=0.95,method=method,formula='y~x') +
-            geom_smooth(aes(x=.data$`log(h-c_hat)`,y=.data$m_lower),span=span,se=FALSE,stat = "smooth",color='black',linetype='solid',size=0.3,alpha=0.95,method=method,formula='y~x') +
+            geom_blank(aes(y=-.data$r_median)) +
+            geom_blank(aes(y=-.data$r_upper)) +
+            geom_blank(aes(y=-.data$r_lower)) +
+            geom_blank(aes(y=-.data$m_upper)) +
+            geom_blank(aes(y=-.data$m_lower)) +
+            geom_smooth(aes(x=.data$`log(h-c_hat)`,y=.data$r_upper),span=span,se=FALSE,stat = "smooth",color='black',linetype='dashed',linewidth=0.5,alpha=0.95,method=method,formula='y~x') +
+            geom_smooth(aes(x=.data$`log(h-c_hat)`,y=.data$r_lower),span=span,se=FALSE,stat = "smooth",color='black',linetype='dashed',linewidth=0.5,alpha=0.95,method=method,formula='y~x') +
+            geom_smooth(aes(x=.data$`log(h-c_hat)`,y=.data$m_upper),span=span,se=FALSE,stat = "smooth",color='black',linetype='solid',linewidth=0.3,alpha=0.95,method=method,formula='y~x') +
+            geom_smooth(aes(x=.data$`log(h-c_hat)`,y=.data$m_lower),span=span,se=FALSE,stat = "smooth",color='black',linetype='solid',linewidth=0.3,alpha=0.95,method=method,formula='y~x') +
             xlab(parse(text=x_lab)) +
             ylab(parse(text=y_lab)) +
-            scale_x_continuous(limits=if(!is.null(args$xlim)) args$xlim else c(NA,NA),expand=expansion(mult=rep(.01,2))) +
-            scale_y_continuous(limits=if(!is.null(args$ylim)) args$ylim else c(NA,NA),expand=expansion(mult=rep(.05,2))) +
-            theme_bdrc()
+            scale_x_continuous(limits=if(!is.null(xlim)) xlim else c(NA,NA),expand=expansion(mult=rep(.01,2))) +
+            scale_y_continuous(limits=if(!is.null(ylim)) ylim else c(NA,NA),expand=expansion(mult=rep(.05,2))) +
+            ggtitle(if(!is.null(title)) title else "Residuals") +
+            theme_bdrc() +
+            theme(plot.title = element_text( vjust = 2 ))
     }else if(type=='r_hat'){
         rhat_dat <- get_rhat_dat(x,param)
         rhat_dat$Rhat[rhat_dat$Rhat<1] <- 1
@@ -270,7 +294,11 @@ plot_fun <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
              scale_color_manual(values=color_palette,name=class(x),labels=param_expr) +
              xlab('Iteration') +
              ylab(parse(text=y_lab)) +
-             theme_bdrc()
+             ggtitle(if(!is.null(title)) title else "Gelman-Rubin statistic") +
+             theme_bdrc() +
+             theme(plot.title = element_text( vjust = 2 ),
+                   axis.title.y = element_text( vjust = 3 ),
+                   plot.margin = ggplot2::margin( t=7, r=7, b=7, l=12, unit = "pt" ) )
     }else if(type=='autocorrelation'){
         auto_dat <- do.call('rbind',lapply(param,function(p) data.frame(lag=x$autocorrelation$lag,param=p,corr=x$autocorrelation[,p])))
         param_expr <- parse(text=get_param_expression(param))
@@ -283,26 +311,35 @@ plot_fun <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
              scale_y_continuous(limits=c(min(auto_dat$corr,-1/11),1),expand=c(0,0)) +
              scale_color_manual(values=color_palette,name=class(x),labels=param_expr) +
              xlab('Lag') +
-             ylab('Autocorrelation') +
-             theme_bdrc()
-    }
-    if(!is.null(args$title)){
-        p <- p + ggtitle(args$title)
+             ylab('Sample autocorrelation') +
+             ggtitle(if(!is.null(title)) title else "Autocorrelation in posterior draws") +
+             theme_bdrc() +
+             theme(plot.title = element_text( vjust = 2 ),
+                   axis.title.y = element_text( vjust = 3 ),
+                   plot.margin = ggplot2::margin( t=7, r=7, b=7, l=12, unit = "pt" ) )
     }
     return(p)
 }
 
 
 #' @importFrom gridExtra arrangeGrob
-#' @importFrom grid textGrob gpar unit
-#' @importFrom ggplot2 theme guides guide_legend
+#' @importFrom grid textGrob gpar unit unit.pmax
+#' @importFrom ggplot2 theme guides guide_legend ggplotGrob
 plot_grob <- function(x,type,transformed=FALSE){
     if(type=='panel'){
         panel_types <- c('rating_curve','residuals','f','sigma_eps')
-        plot_list <- lapply(panel_types,function(ty){
-            plot_fun(x,type=ty,transformed=transformed)
+        grob_list <- lapply(panel_types,function(ty){
+            ggplotGrob(plot_fun(x,type=ty,transformed=transformed))
         })
-        p <- do.call(arrangeGrob,c(plot_list,ncol=round(sqrt(length(panel_types)))))
+        maxHeight <-  unit.pmax( grob_list[[1]]$heights[2:9], grob_list[[2]]$heights[2:9],
+                                 grob_list[[3]]$heights[2:9], grob_list[[4]]$heights[2:9])
+        maxWidth <-  unit.pmax( grob_list[[1]]$widths[2:5], grob_list[[2]]$widths[2:5],
+                                grob_list[[3]]$widths[2:5], grob_list[[4]]$widths[2:5])
+        for(j in 1:4){
+            grob_list[[j]]$heights[2:9] <- as.list(maxHeight)
+            grob_list[[j]]$widths[2:5] <- as.list(maxWidth)
+        }
+        p <- do.call(arrangeGrob,c(grob_list,ncol=round(sqrt(length(panel_types)))))
     }else if(type=='convergence_diagnostics'){
         autocorrelation_plot <- plot_fun(x,type='autocorrelation') +
                                 theme_bdrc(legend.key.size = unit(0.8, "lines"),
@@ -335,7 +372,7 @@ predict_fun <- function(object,newdata=NULL,wide=FALSE){
             newdata <- object$rating_curve$h
         }
     }
-    if(class(newdata) !='numeric'){
+    if(!inherits(newdata,'numeric')){
         stop('newdata must be a vector of type "numeric" or NULL')
     }
     if(any(is.na(newdata))){
@@ -389,7 +426,8 @@ summary.plm0 <- function(object,...){
 #' Autoplot method for discharge rating curves
 #'
 #' Visualize discharge rating curve model objects
-#' @param x an object of class "plm0","plm","gplm0" or "gplm".
+#' @param object an object of class "plm0","plm","gplm0" or "gplm".
+#' @param ... other plotting parameters (not used in this function)
 #' @param type a character denoting what type of plot should be drawn. Defaults to "rating_curve". Possible types are
 #'                    \itemize{
 #'                       \item{"rating_curve"}{ to plot the rating curve.}
@@ -403,12 +441,10 @@ summary.plm0 <- function(object,...){
 #'                    }
 #' @param param a character vector with the parameters to plot. Defaults to NULL and is only used if type is "trace" or "histogram". Allowed values are the parameters given in the model summary of x as well as "hyperparameters" or "latent_parameters" for specific groups of parameters.
 #' @param transformed a logical value indicating whether the quantity should be plotted on a transformed scale used during the Bayesian inference. Defaults to FALSE.
-#' @param ... further arguments passed to other methods. Currently supports:
-#'                     \itemize{
-#'                       \item{"title"}{ a character denoting the title of the plot}
-#'                       \item{"xlim"}{ numeric vector of length 2, denoting the limits on the x axis of the plot. Applicable for types "rating_curve","rating_curve_mean","f","beta",\cr"sigma_eps","residuals".}
-#'                       \item{"ylim"}{ numeric vector of length 2, denoting the limits on the y axis of the plot. Applicable for types "rating_curve","rating_curve_mean","f","beta",\cr"sigma_eps","residuals".}
-#'                     }
+#' @param title a character denoting the title of the plot
+#' @param xlim numeric vector of length 2, denoting the limits on the x axis of the plot. Applicable for types "rating_curve","rating_curve_mean","f","beta","sigma_eps","residuals".
+#' @param ylim numeric vector of length 2, denoting the limits on the y axis of the plot. Applicable for types "rating_curve","rating_curve_mean","f","beta","sigma_eps","residuals".
+#'
 #' @return returns an object of class "ggplot2".
 #' @seealso \code{\link{plm0}}, \code{\link{plm}}, \code{\link{gplm0}} and \code{\link{gplm}} for fitting a discharge rating curve and \code{\link{summary.plm0}}, \code{\link{summary.plm}}, \code{\link{summary.gplm0}} and \code{\link{summary.gplm}} for summaries. It is also useful to look at \code{\link{spread_draws}} and \code{\link{gather_draws}} to work directly with the MCMC samples.
 #' @examples
@@ -430,14 +466,15 @@ summary.plm0 <- function(object,...){
 #' @describeIn autoplot.plm0 Autoplot method for plm0
 #' @importFrom ggplot2 autoplot
 #' @export
-autoplot.plm0 <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
-    plot_fun(x,type=type,param=param,transformed=transformed,...)
+autoplot.plm0 <- function(object,...,type='rating_curve',param=NULL,transformed=FALSE,title=NULL,xlim=NULL,ylim=NULL){
+    plot_fun(object,type=type,param=param,transformed=transformed,title=title,xlim=xlim,ylim=ylim)
 }
 
 #' Plot method for discharge rating curves
 #'
 #' Visualize discharge rating curve model objects
 #' @param x object of class "plm0", "plm", "gplm0" or "gplm".
+#' @param ... other plotting parameters (not used in this function)
 #' @param type a character denoting what type of plot should be drawn. Defaults to "rating_curve". Possible types are
 #'                    \itemize{
 #'                       \item{"rating_curve"}{ to plot the rating curve.}
@@ -452,12 +489,10 @@ autoplot.plm0 <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...
 #'                    }
 #' @param param a character vector with the parameters to plot. Defaults to NULL and is only used if type is "trace" or "histogram". Allowed values are the parameters given in the model summary of x as well as "hyperparameters" or "latent_parameters" for specific groups of parameters.
 #' @param transformed a logical value indicating whether the quantity should be plotted on a transformed scale used during the Bayesian inference. Defaults to FALSE.
-#' @param ... further arguments passed to other methods. Currently supports:
-#'                     \itemize{
-#'                       \item{"title"}{ a character denoting the title of the plot}
-#'                       \item{"xlim"}{ numeric vector of length 2, denoting the limits on the x axis of the plot. Applicable for types "rating_curve","rating_curve_mean","f","beta"\cr,"sigma_eps","residuals".}
-#'                       \item{"ylim"}{ numeric vector of length 2, denoting the limits on the y axis of the plot. Applicable for types "rating_curve","rating_curve_mean","f","beta"\cr,"sigma_eps","residuals".}
-#'                     }
+#' @param title a character denoting the title of the plot
+#' @param xlim numeric vector of length 2, denoting the limits on the x axis of the plot. Applicable for types "rating_curve","rating_curve_mean","f","beta","sigma_eps","residuals".
+#' @param ylim numeric vector of length 2, denoting the limits on the y axis of the plot. Applicable for types "rating_curve","rating_curve_mean","f","beta","sigma_eps","residuals".
+#'
 #' @return No return value, called for side effects.
 #' @seealso \code{\link{plm0}}, \code{\link{plm}}, \code{\link{gplm0}} and \code{\link{gplm}} for fitting a discharge rating curve and \code{\link{summary.plm0}}, \code{\link{summary.plm}}, \code{\link{summary.gplm0}} and \code{\link{summary.gplm}} for summaries. It is also useful to look at \code{\link{spread_draws}} and \code{\link{gather_draws}} to work directly with the MCMC samples.
 #' @examples
@@ -480,10 +515,10 @@ autoplot.plm0 <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...
 #' @export
 #' @importFrom grid grid.draw
 #' @importFrom ggplot2 autoplot
-plot.plm0 <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
+plot.plm0 <- function(x,...,type='rating_curve',param=NULL,transformed=FALSE,title=NULL,xlim=NULL,ylim=NULL){
     grob_types <- c('panel','convergence_diagnostics')
     if(is.null(type) || !(type%in%grob_types)){
-        p <- autoplot(x,type=type,param=param,transformed=transformed,...)
+        p <- autoplot(x,type=type,param=param,transformed=transformed,title=title,xlim=xlim,ylim=ylim)
         print(p)
     }else{
         p <- plot_grob(x,type=type,transformed=transformed)
@@ -510,8 +545,8 @@ plot.plm0 <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
 #' }
 #' @describeIn predict.plm0 Predict method for plm0
 #' @export
-predict.plm0 <- function(object,newdata=NULL,wide=FALSE,...){
-    predict_fun(object,newdata,wide)
+predict.plm0 <- function(object,...,newdata=NULL,wide=FALSE){
+    predict_fun(object,newdata=newdata,wide=wide)
 }
 
 #' @describeIn print.plm0 Print method for plm
@@ -529,18 +564,18 @@ summary.plm <- function(object,...){
 
 #' @describeIn autoplot.plm0 Autoplot method for plm
 #' @export
-autoplot.plm <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
-    plot_fun(x,type=type,param=param,transformed=transformed,...)
+autoplot.plm <- function(object,...,type='rating_curve',param=NULL,transformed=FALSE,title=NULL,xlim=NULL,ylim=NULL){
+    plot_fun(object,type=type,param=param,transformed=transformed,title=title,xlim=xlim,ylim=ylim)
 }
 
 #' @describeIn plot.plm0 Plot method for plm
 #' @export
 #' @importFrom grid grid.draw
 #' @importFrom ggplot2 autoplot
-plot.plm <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
+plot.plm <- function(x,...,type='rating_curve',param=NULL,transformed=FALSE,title=NULL,xlim=NULL,ylim=NULL){
     grob_types <- c('panel','convergence_diagnostics')
     if(is.null(type) || !(type%in%grob_types)){
-        p <- autoplot(x,type=type,param=param,transformed=transformed,...)
+        p <- autoplot(x,type=type,param=param,transformed=transformed,title=title,xlim=xlim,ylim=ylim)
         print(p)
     }else{
         p <- plot_grob(x,type=type,transformed=transformed)
@@ -550,8 +585,8 @@ plot.plm <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
 
 #' @describeIn predict.plm0 Predict method for plm
 #' @export
-predict.plm <- function(object,newdata=NULL,wide=FALSE,...){
-    predict_fun(object,newdata,wide)
+predict.plm <- function(object,...,newdata=NULL,wide=FALSE){
+    predict_fun(object,newdata=newdata,wide=wide)
 }
 
 #' @describeIn print.plm0 Print method for gplm0
@@ -568,18 +603,18 @@ summary.gplm0 <- function(object,...){
 
 #' @describeIn autoplot.plm0 Autoplot method for gplm0
 #' @export
-autoplot.gplm0 <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
-    plot_fun(x,type=type,param=param,transformed=transformed,...)
+autoplot.gplm0 <- function(object,...,type='rating_curve',param=NULL,transformed=FALSE,title=NULL,xlim=NULL,ylim=NULL){
+    plot_fun(object,type=type,param=param,transformed=transformed,title=title,xlim=xlim,ylim=ylim)
 }
 
 #' @describeIn plot.plm0 Plot method for gplm0
 #' @export
 #' @importFrom grid grid.draw
 #' @importFrom ggplot2 autoplot
-plot.gplm0 <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
+plot.gplm0 <- function(x,...,type='rating_curve',param=NULL,transformed=FALSE,title=NULL,xlim=NULL,ylim=NULL){
     grob_types <- c('panel','convergence_diagnostics')
     if(is.null(type) || !(type%in%grob_types)){
-        p <- autoplot(x,type=type,param=param,transformed=transformed,...)
+        p <- autoplot(x,type=type,param=param,transformed=transformed,title=title,xlim=xlim,ylim=ylim)
         print(p)
     }else{
         p <- plot_grob(x,type=type,transformed=transformed)
@@ -589,8 +624,8 @@ plot.gplm0 <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
 
 #' @describeIn predict.plm0 Predict method for gplm0
 #' @export
-predict.gplm0 <- function(object,newdata=NULL,wide=FALSE,...){
-    predict_fun(object,newdata,wide)
+predict.gplm0 <- function(object,...,newdata=NULL,wide=FALSE){
+    predict_fun(object,newdata=newdata,wide=wide)
 }
 
 #' @describeIn print.plm0 Print method for gplm
@@ -607,18 +642,18 @@ summary.gplm <- function(object,...){
 
 #' @describeIn autoplot.plm0 Autoplot method for gplm
 #' @export
-autoplot.gplm <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
-    plot_fun(x,type=type,param=param,transformed=transformed,...)
+autoplot.gplm <- function(object,...,type='rating_curve',param=NULL,transformed=FALSE,title=NULL,xlim=NULL,ylim=NULL){
+    plot_fun(object,type=type,param=param,transformed=transformed,title=title,xlim=xlim,ylim=ylim)
 }
 
 #' @describeIn plot.plm0 Plot method for gplm
 #' @export
 #' @importFrom grid grid.draw
 #' @importFrom ggplot2 autoplot
-plot.gplm <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
+plot.gplm <- function(x,...,type='rating_curve',param=NULL,transformed=FALSE,title=NULL,xlim=NULL,ylim=NULL){
     grob_types <- c('panel','convergence_diagnostics')
     if(is.null(type) || !(type%in%grob_types)){
-        p <- autoplot(x,type=type,param=param,transformed=transformed,...)
+        p <- autoplot(x,type=type,param=param,transformed=transformed,title=title,xlim=xlim,ylim=ylim)
         print(p)
     }else{
         p <- plot_grob(x,type=type,transformed=transformed)
@@ -628,6 +663,6 @@ plot.gplm <- function(x,type='rating_curve',param=NULL,transformed=FALSE,...){
 
 #' @describeIn predict.plm0 Predict method for gplm
 #' @export
-predict.gplm <- function(object,newdata=NULL,wide=FALSE,...){
-    predict_fun(object,newdata,wide)
+predict.gplm <- function(object,...,newdata=NULL,wide=FALSE){
+    predict_fun(object,newdata=newdata,wide=wide)
 }
